@@ -54,7 +54,7 @@ be created for the lifetime of the spawner.
 use strict;
 use warnings;
 
-our $VERSION = '2.08';
+our $VERSION = '2.11';
 
 use Carp qw/ croak /;
 use Danga::Socket ();
@@ -607,7 +607,7 @@ sub _supervise {
     my $fileno = shift;
     open my $reader, '<&=', $fileno or die "failed to open pipe: $!\n";
 
-    my $startup_data = <$reader>; # need this now, so allow blocking read
+    chomp(my $startup_data = <$reader>); # need this now, so allow blocking read
     my $startup_params = _unserialize($startup_data);
 
     @INC = @{ $startup_params->{inc} };
@@ -636,9 +636,14 @@ sub _supervise {
 
     # set nonblocking since these commands come any time
     IO::Handle::blocking($reader, 0);
+    my $read_buf = '';
     my $handler = sub {
         while (my $line = <$reader>) {
-            my $slots = _unserialize($line);
+            $read_buf .= $line;
+            last unless $line =~ /\n$/;
+            chomp($read_buf);
+            my $slots = _unserialize($read_buf);
+            $read_buf = '';
             push @open_slots, @$slots;
         }
     };
@@ -709,14 +714,14 @@ sub _do_work {
 
     my $params = $slot->[SLOT_PARAMS];
     my $worker_class = $params->{class};
+    $0 = sprintf "%s #%d", $worker_class, $slot->[SLOT_NUM];
+
     my $worker = $worker_class->new($slot->[SLOT_NUM], $params->{config}, gearman_servers());
 
     die "failed to create $worker_class object" unless $worker;
 
     $worker->job_servers(@{ $self->{gearmand} });
 
-    $0 = sprintf "%s #%d", $worker_class, $slot->[SLOT_NUM];
-
     # each worker gets a unique function so we can ping it in wait_until_all_ready
     $worker->register_function(_ping_name($slot->[SLOT_ID]) => sub {
         if ($worker->can('unregister_function')) {
@@ -758,7 +763,7 @@ sub _serialize {
 }
 
 sub _unserialize {
-    chomp(my $frozen = shift);
+    my $frozen = shift;
     return thaw pack 'h*', $frozen;
 }
 
